@@ -21,12 +21,27 @@ double = numpy.double
 
 # The default kind for undeclared variables
 default_kind = 'double'
+if sys.version_info[0] < 3:
+    int_ = int
+    long_ = long
+else:
+    int_ = numpy.int32
+    long_ = numpy.int64
 
-type_to_kind = {bool: 'bool', int: 'int', long: 'long', float: 'float',
-                double: 'double', complex: 'complex', str: 'str'}
-kind_to_type = {'bool': bool, 'int': int, 'long': long, 'float': float,
-                'double': double, 'complex': complex, 'str': str}
+type_to_kind = {bool: 'bool', int_: 'int', long_: 'long', float: 'float',
+                double: 'double', complex: 'complex', bytes: 'bytes'}
+kind_to_type = {'bool': bool, 'int': int_, 'long': long_, 'float': float,
+                'double': double, 'complex': complex, 'bytes': bytes}
 kind_rank = ['bool', 'int', 'long', 'float', 'double', 'complex', 'none']
+scalar_constant_types = [bool, int_, long, float, double, complex, bytes]
+
+# Final corrections for Python 3 (mainly for PyTables needs)
+if sys.version_info[0] > 2:
+    type_to_kind[str] = 'str'
+    kind_to_type['str'] = str
+    scalar_constant_types.append(str)
+scalar_constant_types = tuple(scalar_constant_types)
+
 
 from numexpr import interpreter
 
@@ -86,15 +101,15 @@ def allConstantNodes(args):
 
 def isConstant(ex):
     "Returns True if ex is a constant scalar of an allowed type."
-    return isinstance(ex, (bool, int, long, float, double, complex, str))
+    return isinstance(ex, scalar_constant_types)
 
 def commonKind(nodes):
     node_kinds = [node.astKind for node in nodes]
-    str_count = node_kinds.count('str')
+    str_count = node_kinds.count('bytes') + node_kinds.count('str')
     if 0 < str_count < len(node_kinds):  # some args are strings, but not all
         raise TypeError("strings can only be operated with strings")
     if str_count > 0:  # if there are some, all of them must be
-        return 'str'
+        return 'bytes'
     n = -1
     for x in nodes:
         n = max(n, kind_rank.index(x.astKind))
@@ -104,31 +119,32 @@ max_int32 = 2147483647
 min_int32 = -max_int32 - 1
 
 def bestConstantType(x):
-    if isinstance(x, str):  # ``numpy.string_`` is a subclass of ``str``
-        return str
-    # ``long`` objects are kept as is to allow the user to force
-    # promotion of results by using long constants, e.g. by operating
-    # a 32-bit array with a long (64-bit) constant.
-    if isinstance(x, (long, numpy.int64)):
-        return long
-    # ``double`` objects are kept as is to allow the user to force
-    # promotion of results by using double constants, e.g. by operating
-    # a float (32-bit) array with a double (64-bit) constant.
-    if isinstance(x, double):
-        return double
+    # ``numpy.string_`` is a subclass of ``bytes``
+    if isinstance(x, (bytes, str)):
+        return bytes
     # Numeric conversion to boolean values is not tried because
     # ``bool(1) == True`` (same for 0 and False), so 0 and 1 would be
     # interpreted as booleans when ``False`` and ``True`` are already
     # supported.
     if isinstance(x, (bool, numpy.bool_)):
         return bool
+    # ``long`` objects are kept as is to allow the user to force
+    # promotion of results by using long constants, e.g. by operating
+    # a 32-bit array with a long (64-bit) constant.
+    if isinstance(x, (long_, numpy.int64)):
+        return long_
+    # ``double`` objects are kept as is to allow the user to force
+    # promotion of results by using double constants, e.g. by operating
+    # a float (32-bit) array with a double (64-bit) constant.
+    if isinstance(x, double):
+        return double
     if isinstance(x, (int, numpy.integer)):
         # Constants needing more than 32 bits are always
         # considered ``long``, *regardless of the platform*, so we
         # can clearly tell 32- and 64-bit constants apart.
         if not (min_int32 <= x <= max_int32):
-            return long
-        return int
+            return long_
+        return int_
     # The duality of float and double in Python avoids that we have to list
     # ``double`` too.
     for converter in float, complex:
@@ -204,13 +220,13 @@ def sum_func(a, axis=None):
     axis = encode_axis(axis)
     if isinstance(a, ConstantNode):
         return a
-    if isinstance(a, (bool, int, long, float, double, complex)):
+    if isinstance(a, (bool, int_, long_, float, double, complex)):
         a = ConstantNode(a)
     return FuncNode('sum', [a, axis], kind=a.astKind)
 
 def prod_func(a, axis=None):
     axis = encode_axis(axis)
-    if isinstance(a, (bool, int, long, float, double, complex)):
+    if isinstance(a, (bool, int_, long_, float, double, complex)):
         a = ConstantNode(a)
     if isinstance(a, ConstantNode):
         return a
@@ -252,8 +268,8 @@ def pow_op(a, b):
             # Optimize all integral and half integral powers in [-RANGE, RANGE]
             # Note: for complex numbers RANGE could be larger.
             if (int(2*x) == 2*x) and (-RANGE <= abs(x) <= RANGE):
-                n = int(abs(x))
-                ishalfpower = int(abs(2*x)) % 2
+                n = int_(abs(x))
+                ishalfpower = int_(abs(2*x)) % 2
                 def multiply(x, y):
                     if x is None: return y
                     return OpNode('mul', [x, y])
@@ -380,18 +396,19 @@ class ExpressionNode(object):
 
     # The next check is commented out. See #24 for more info.
 
-    # def __nonzero__(self):
-    #     raise TypeError("You can't use Python's standard boolean operators in "
-    #                     "NumExpr expressions. You should use their bitwise "
-    #                     "counterparts instead: '&' instead of 'and', "
-    #                     "'|' instead of 'or', and '~' instead of 'not'.")
+    def __nonzero__(self):
+         raise TypeError("You can't use Python's standard boolean operators in "
+                         "NumExpr expressions. You should use their bitwise "
+                         "counterparts instead: '&' instead of 'and', "
+                         "'|' instead of 'or', and '~' instead of 'not'.")
 
     __add__ = __radd__ = binop('add')
     __sub__ = binop('sub')
     __rsub__ = binop('sub', reversed=True)
     __mul__ = __rmul__ = binop('mul')
-    __div__ = div_op
-    __rdiv__ = binop('div', reversed=True)
+    if sys.version_info[0] < 3:
+        __div__ = div_op
+        __rdiv__ = binop('div', reversed=True)
     __truediv__ = truediv_op
     __rtruediv__ = rtruediv_op
     __pow__ = pow_op
