@@ -254,7 +254,7 @@
         case OP_MUL_III: VEC_ARG2(i_dest = i1 * i2);
         case OP_DIV_III: VEC_ARG2(i_dest = i2 ? (i1 / i2) : 0);
         case OP_POW_III: VEC_ARG2(i_dest = (i2 < 0) ? (1 / i1) : (int)pow((double)i1, i2));
-        case OP_MOD_III: VEC_ARG2(i_dest = i1 % i2);
+        case OP_MOD_III: VEC_ARG2(i_dest = i2 ? (i1 % i2) : 0);
 
         case OP_WHERE_IBII: VEC_ARG3(i_dest = b1 ? i2 : i3);
 
@@ -268,7 +268,7 @@
         case OP_MUL_LLL: VEC_ARG2(l_dest = l1 * l2);
         case OP_DIV_LLL: VEC_ARG2(l_dest = l2 ? (l1 / l2) : 0);
         case OP_POW_LLL: VEC_ARG2(l_dest = (l2 < 0) ? (1 / l1) : (long long)pow((long double)l1, (long double)l2));
-        case OP_MOD_LLL: VEC_ARG2(l_dest = l1 % l2);
+        case OP_MOD_LLL: VEC_ARG2(l_dest = l2 ? (l1 % l2) : 0);
 
         case OP_WHERE_LBLL: VEC_ARG3(l_dest = b1 ? l2 : l3);
 
