@@ -20,7 +20,8 @@ from numpy import (
     sin, cos, tan, arcsin, arccos, arctan, arctan2,
     sinh, cosh, tanh, arcsinh, arccosh, arctanh,
     log, log1p, log10, exp, expm1)
-from numpy.testing import *
+from numpy.testing import (assert_equal, assert_array_equal,
+    assert_array_almost_equal, assert_allclose)
 from numpy import shape, allclose, array_equal, ravel, isnan, isinf
 
 import numexpr
@@ -36,6 +37,8 @@ double = numpy.double
 minimum_numpy_version = "1.6"
 
 class test_numexpr(TestCase):
+    """Testing with 1 thread"""
+    nthreads = 1
 
     def setUp(self):
         numexpr.set_num_threads(self.nthreads)
@@ -159,10 +162,6 @@ class test_numexpr(TestCase):
                      (b'add_ddd', b'r0', b'r0', b'c2[2.0]')])
 
 
-class test_numexpr1(test_numexpr):
-    """Testing with 1 thread"""
-    nthreads = 1
-
 class test_numexpr2(test_numexpr):
     """Testing with 2 threads"""
     nthreads = 2
@@ -326,6 +325,8 @@ for op in list('+-*/%') + ['**']:
     optests.append("(a+1) %s 2" % op)
     optests.append("(a+1) %s -1" % op)
     optests.append("(a+1) %s 0.5" % op)
+    # Check divisions and modulus by zero (see ticket #107)
+    optests.append("(a+1) %s 0" % op)
 tests.append(('OPERATIONS', optests))
 
 cmptests = []
@@ -392,14 +393,11 @@ def test_expressions():
         this_locals = locals()
         def method():
             # We don't want to listen at RuntimeWarnings like
-            # "overflows" or "divide by zero".  Feel free to expand
-            # the range for this filter, if needed.
-            if dtype.__name__ == "float32" or 'arctanh' in expr:
-                warnings.simplefilter("ignore")
-                npval = eval(expr, globals(), this_locals)
-                warnings.simplefilter("always")
-            else:
-                npval = eval(expr, globals(), this_locals)
+            # "overflows" or "divide by zero" in plain eval().
+            warnings.simplefilter("ignore")
+            npval = eval(expr, globals(), this_locals)
+            warnings.simplefilter("always")
+            npval = eval(expr, globals(), this_locals)
             try:
                 neval = evaluate(expr, local_dict=this_locals,
                                  optimization=optimization)
@@ -733,7 +731,8 @@ def print_versions():
     print "VML available?     %s" % use_vml
     if use_vml:
         print "VML/MKL version:   %s" % numexpr.get_vml_version()
-    print 'Detected cores:    %s' % numexpr.ncores
+    print ("Number of threads used by default: %d "
+           "(out of %d detected cores)" % (numexpr.nthreads, numexpr.ncores))
     print '-=' * 38
 
 
@@ -743,7 +742,7 @@ def test():
     """
 
     print_versions()
-    unittest.TextTestRunner().run(suite())
+    return unittest.TextTestRunner().run(suite())
 test.__test__ = False
 
 
@@ -767,7 +766,7 @@ def suite():
         add_method(func)
 
     for n in range(niter):
-        theSuite.addTest(unittest.makeSuite(test_numexpr1))
+        theSuite.addTest(unittest.makeSuite(test_numexpr))
         theSuite.addTest(unittest.makeSuite(test_numexpr2))
         theSuite.addTest(unittest.makeSuite(test_evaluate))
         theSuite.addTest(unittest.makeSuite(TestExpressions))
